clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Gaussian Losses
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

load FireLosses.mat

alpha=.01;

mu=mean(FireLosses);
sigma=std(FireLosses);
z_99=norminv(.99,0,1); %obtain the extreme 99% quantile of loss

GaussianVaR=mu+sigma*z_99

GaussianES=mu+sigma*normpdf(z_99,0,1)/alpha

% Note that assuming a Gaussian distribution is a very simplistic
% approach: losses are always positive, we only consider losses above 
% 300'000 DKK ...

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Kernel approach
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

z=FireLosses;

opt=optimset('Display','iter','HessUpdate','steepdesc');
guess=GaussianVaR;
NonParamVaR=fminsearch(@obj_fct,guess,opt,z)
% fminsearch is not a nice optimizer to use, but in this case while the
% other standard optimizers fail, fminsearch reaches convergence


%Function minimized is the same as in previous TP--Idea is to minimize 
%fitting errors

T=length(FireLosses);
h=std(FireLosses)*T^(-.2);
NonParamES=mean(FireLosses.*normcdf((FireLosses-NonParamVaR)/h))/alpha
%This is an implementation of last formula in page A.X-83


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% EVT approach (POT method)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Although you are asked to use only the Peaks-Over-Threshold method,
% be aware that there are two approaches to EVT, the other one being
% the modeling of extremes.

% u=10^7;
u=2*10^7;

z=FireLosses(FireLosses>u); %choose losses above treshold u

% Method of Moments estimation of parameters:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m=mean(z);
S2=var(z,1);  % empirical variance

xi=.5*(1-(m-u)^2/S2); %These formulas are in A.XI-96
sigma=.5*(m-u)*((m-u)^2/S2+1);

N=length(z);
sigma_tilde=sigma*(N/T)^xi;

mu_tilde=u-sigma_tilde/xi*((N/T)^(-xi)-1);

POTVaR=mu_tilde+sigma_tilde/xi*(alpha^(-xi)-1) %Forumula in A.XI-99

POTES=POTVaR-sigma_tilde/(xi-1)*alpha^(-xi)

% ML estimation of GPD parameters:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
estimates=fminsearch(@GPDLogLikelihood,[xi sigma],opt,z-u);
xi_ML=estimates(1);
sigma_ML=estimates(2);

sigma_tilde_ML=sigma_ML*(N/T)^xi_ML;

mu_tilde_ML=u-sigma_tilde_ML/xi_ML*((N/T)^(-xi_ML)-1); %Forumula in A.XI-99

POTVaR_ML=mu_tilde_ML+sigma_tilde_ML/xi_ML*(alpha^(-xi_ML)-1)

POTES_ML=POTVaR_ML-sigma_tilde_ML/(xi_ML-1)*alpha^(-xi_ML)


% gpfit(X) returns maximum likelihood estimates of the parameters for the
% generalized Pareto (GP) distribution given the data in X.
% (It may not be available on your version of the Statistics Toolbox.)

% Note how the different assumptions and estimation methods affect the
% results.